﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.ComplexEventProcessing.Adapters;
using System.Threading;
using Microsoft.ComplexEventProcessing;
using System.Diagnostics;

namespace DemoConsole.Adapters
{
    public class ConsoleOutputAdapter<TPayload>
        : TypedPointOutputAdapter<TPayload>
    {
        ConsoleOutputAdapterConfig _config;

        public ConsoleOutputAdapter(ConsoleOutputAdapterConfig config)
        {
            _config = config;
        }

        public override void Start()
        {
            new Thread(ConsumeEvents).Start();
        }

        public override void Resume()
        {
            new Thread(ConsumeEvents).Start();
        }

        void ConsumeEvents()
        {
            var currentEvent = default(PointEvent<TPayload>);

            try
            {
                while (true)
                {
                    if (AdapterState.Stopping == AdapterState)
                    {
                        Stopped();
                        return;
                    }

                    // Dequeue the event. If the dequeue fails, then the adapter state is suspended
                    // or stopping. Assume the former and call Ready() to indicate
                    // readiness to be resumed, and exit the thread.
                    if (DequeueOperationResult.Empty == Dequeue(out currentEvent))
                    {
                        Ready();
                        return;
                    }

                    if (currentEvent.EventKind == EventKind.Insert)
                    {
                        Console.WriteLine(_config.OutputMessageFormatString, currentEvent.Payload);
                    } else if (currentEvent.EventKind == EventKind.Cti)
                    {
                        //do nothing
                    }

                    // Every received event needs to be released.
                    ReleaseEvent(ref currentEvent);
                }
            } catch (Exception e)
            {
                Debug.WriteLine(e.ToString());
            }
        }
    }
}
