﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.ComplexEventProcessing.Adapters;
using Microsoft.ComplexEventProcessing;

namespace DemoConsole.Adapters
{
    public class RandomPayloadInputAdapterFactory :
        ITypedInputAdapterFactory<RandomPayloadConfig>,
        ITypedDeclareAdvanceTimeProperties<RandomPayloadConfig>
    {
        public InputAdapterBase Create<TPayload>(RandomPayloadConfig configInfo, EventShape eventShape)
        {
            if (eventShape != EventShape.Point) throw new InvalidOperationException("Point only.");

            return new RandomPayloadPointInputAdapter(configInfo);
        }

        public void Dispose()
        {

        }

        /// <summary>
        /// Specifies the CTI behavior of this adapter. This method will be called whenever
        /// a new adapter instance is needed, with the respective configuration and event type.
        /// </summary>
        /// <typeparam name="TPayload">Type of the payload.</typeparam>
        /// <param name="configInfo">Configuration passed from the query binding.</param>
        /// <param name="eventShape">Event shape requested from the query binding.</param>
        /// <returns>An instance of AdapterAdvanceTimeSettings.</returns>
        public AdapterAdvanceTimeSettings DeclareAdvanceTimeProperties<TPayload>(RandomPayloadConfig configInfo, EventShape eventShape)
        {
            // Inject a CTI one tick _after_ (negative delay) each event and
            // drop violating events. This ensures maximum liveliness, since
            // each event is immediately committed.  Note that this will drop
            // duplicate events (events collocated in time) when enqueued by
            // the input adapter.
            return new AdapterAdvanceTimeSettings(
                new AdvanceTimeGenerationSettings(configInfo.CtiFrequency, TimeSpan.FromTicks(-1)),
                AdvanceTimePolicy.Drop);
        }
    }
}
