﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.ComplexEventProcessing.Adapters;
using System.Timers;
using Microsoft.ComplexEventProcessing;

namespace DemoConsole.Adapters
{
    public class RandomPayloadPointInputAdapter : TypedPointInputAdapter<EventPayload>
    {
        RandomPayloadConfig _config;
        Timer _timer;

        public RandomPayloadPointInputAdapter(RandomPayloadConfig config)
        {
            _config = config;
            _timer = new Timer(_config.SleepTime);
            _timer.AutoReset = true;
            _timer.Elapsed += ProduceEvent;
        }

        public override void Start()
        {
            _timer.Start();
        }

        public override void Resume()
        {
            _timer.Start();
        }

        void ProduceEvent(object sender, ElapsedEventArgs e)
        {
            // Check for the stopping state and stop if necessary.
            if (AdapterState.Stopping == AdapterState)
            {
                _timer.Stop();
                Stopped();
                return;
            }

            // Since this method is called from a thread that is independend of
            // Start() and Resume(), we need to make sure that the adapter is
            // actually running.
            if (AdapterState.Running != AdapterState)
            {
                // Throw away the current event.
                return;
            }

            PointEvent<EventPayload> currEvent = CreateInsertEvent();
            if (null == currEvent)
            {
                // Throw away the current event.
                return;
            }

            SetRandomEventPayload(currEvent);

            if (EnqueueOperationResult.Full == Enqueue(ref currEvent))
            {
                ReleaseEvent(ref currEvent);
                Ready();
                return;
            }
        }

        static Random _random = new Random();
        void SetRandomEventPayload(PointEvent<EventPayload> e)
        {
            e.Payload =
                new EventPayload
                {
                    Region = ((Region)_random.Next(4)).ToString(),
                    Value = _random.NextDouble()
                };
            e.StartTime = DateTime.Now;
        }
    }
}
