﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.ComplexEventProcessing;
using DemoConsole.Adapters;
using Microsoft.ComplexEventProcessing.Linq;

namespace DemoConsole
{
    class Program
    {
        static void Main(string[] args)
        {
            using (var server =
                Server.Create("Default"))
                //for shared server scenarios
                //Server.Connect(new EndpointAddress(@"http://localhost/StreamInsight/Default")))
            {
                //for shared server scenarios
                if (server.Applications.ContainsKey("Msdn"))
                {
                    server.Applications["Msdn"].Delete();
                }
                var app = server.CreateApplication("Msdn");

                var inputConfig =
                    new RandomPayloadConfig
                    {
                        SleepTime = 200
                    };

                var inputStream =
                    CepStream<EventPayload>.Create(
                        "input", typeof(RandomPayloadInputAdapterFactory), inputConfig, EventShape.Point);

                //Simple filtering query
                var filtered =
                    from i in inputStream
                    where i.Value > 0.5
                    select i;

                //Simple aggregation over time
                var aggregated =
                    from i in inputStream
                        .TumblingWindow(TimeSpan.FromSeconds(5), HoppingWindowOutputPolicy.ClipToWindowEnd)
                    select new { Avg = i.Avg(p => p.Value)};

                //Simple grouping query
                var payloadByRegion =
                    from i in inputStream
                    group i by i.Region into byRegion
                    from c in byRegion.HoppingWindow(
                        TimeSpan.FromMinutes(1),
                        TimeSpan.FromSeconds(2), HoppingWindowOutputPolicy.ClipToWindowEnd)
                    select new { Region = byRegion.Key, Sum = c.Sum(p => p.Value) };

                //TopK
                var highestRegion = //uses groupBy query above
                    (from i in payloadByRegion.SnapshotWindow(SnapshotWindowOutputPolicy.Clip)
                     from sumByRegion in i
                     orderby sumByRegion.Sum descending
                     select sumByRegion).Take(1);                    

                //Simple count window
                //Count window only works with user defined aggregates                
                var sumEvery5 =
                    from i in inputStream
                        .CountByStartTimeWindow(5, CountWindowOutputPolicy.PointAlignToWindowEnd)
                    select new { Sum = i.EveryOtherSum(p => p.Value) };

                //Select query to run
                var template = filtered; 
                //var template = aggregated;
                //var template = payloadByRegion;                
                //var template = highestRegion;
                //var template = sumEvery5;

                var query =
                    template.ToQuery(app, "query", "description",
                        typeof(ConsoleOutputAdapterFactory),
                        new ConsoleOutputAdapterConfig
                        {
                            OutputMessageFormatString = "Received event: {0}"
                        }, EventShape.Point, StreamEventOrder.FullyOrdered);

                Console.WriteLine("Starting...");
                query.Start();
                
                Console.WriteLine("Started. Press Enter to Stop.");
                Console.ReadLine();
                
                query.Stop();
                Console.WriteLine("Stopped. Press Enter to quit.");
                Console.ReadLine();
            }
        }
    }
}
